"""The base class and interface for all formatting plugins."""
from __future__ import print_function


class BaseFormatter(object):
    """Class defining the formatter interface.

    .. attribute:: options

        The options parsed from both configuration files and the command-line.

    .. attribute:: filename

        If specified by the user, the path to store the results of the run.

    .. attribute:: output_fd

        Initialized when the :meth:`start` is called. This will be a file
        object opened for writing.

    .. attribute:: newline

        The string to add to the end of a line. This is only used when the
        output filename has been specified.
    """

    def __init__(self, options):
        """Initialize with the options parsed from config and cli.

        This also calls a hook, :meth:`after_init`, so subclasses do not need
        to call super to call this method.

        :param optparse.Values options:
            User specified configuration parsed from both configuration files
            and the command-line interface.
        """
        self.options = options
        self.filename = options.output_file
        self.output_fd = None
        self.newline = "\n"
        self.after_init()

    def after_init(self):
        """Initialize the formatter further."""
        pass

    def beginning(self, filename):
        """Notify the formatter that we're starting to process a file.

        :param str filename:
            The name of the file that Flake8 is beginning to report results
            from.
        """
        pass

    def finished(self, filename):
        """Notify the formatter that we've finished processing a file.

        :param str filename:
            The name of the file that Flake8 has finished reporting results
            from.
        """
        pass

    def start(self):
        """Prepare the formatter to receive input.

        This defaults to initializing :attr:`output_fd` if :attr:`filename`
        """
        if self.filename:
            self.output_fd = open(self.filename, "a")

    def handle(self, error):
        """Handle an error reported by Flake8.

        This defaults to calling :meth:`format`, :meth:`show_source`, and
        then :meth:`write`. To extend how errors are handled, override this
        method.

        :param error:
            This will be an instance of
            :class:`~flake8.style_guide.Violation`.
        :type error:
            flake8.style_guide.Violation
        """
        line = self.format(error)
        source = self.show_source(error)
        self.write(line, source)

    def format(self, error):
        """Format an error reported by Flake8.

        This method **must** be implemented by subclasses.

        :param error:
            This will be an instance of
            :class:`~flake8.style_guide.Violation`.
        :type error:
            flake8.style_guide.Violation
        :returns:
            The formatted error string.
        :rtype:
            str
        """
        raise NotImplementedError(
            "Subclass of BaseFormatter did not implement" " format."
        )

    def show_statistics(self, statistics):
        """Format and print the statistics."""
        for error_code in statistics.error_codes():
            stats_for_error_code = statistics.statistics_for(error_code)
            statistic = next(stats_for_error_code)
            count = statistic.count
            count += sum(stat.count for stat in stats_for_error_code)
            self._write(
                "{count:<5} {error_code} {message}".format(
                    count=count,
                    error_code=error_code,
                    message=statistic.message,
                )
            )

    def show_benchmarks(self, benchmarks):
        """Format and print the benchmarks."""
        # NOTE(sigmavirus24): The format strings are a little confusing, even
        # to me, so here's a quick explanation:
        # We specify the named value first followed by a ':' to indicate we're
        # formatting the value.
        # Next we use '<' to indicate we want the value left aligned.
        # Then '10' is the width of the area.
        # For floats, finally, we only want only want at most 3 digits after
        # the decimal point to be displayed. This is the precision and it
        # can not be specified for integers which is why we need two separate
        # format strings.
        float_format = "{value:<10.3} {statistic}".format
        int_format = "{value:<10} {statistic}".format
        for statistic, value in benchmarks:
            if isinstance(value, int):
                benchmark = int_format(statistic=statistic, value=value)
            else:
                benchmark = float_format(statistic=statistic, value=value)
            self._write(benchmark)

    def show_source(self, error):
        """Show the physical line generating the error.

        This also adds an indicator for the particular part of the line that
        is reported as generating the problem.

        :param error:
            This will be an instance of
            :class:`~flake8.style_guide.Violation`.
        :type error:
            flake8.style_guide.Violation
        :returns:
            The formatted error string if the user wants to show the source.
            If the user does not want to show the source, this will return
            ``None``.
        :rtype:
            str
        """
        if not self.options.show_source or error.physical_line is None:
            return ""

        # Because column numbers are 1-indexed, we need to remove one to get
        # the proper number of space characters.
        pointer = (" " * (error.column_number - 1)) + "^"
        # Physical lines have a newline at the end, no need to add an extra
        # one
        return error.physical_line + pointer

    def _write(self, output):
        """Handle logic of whether to use an output file or print()."""
        if self.output_fd is not None:
            self.output_fd.write(output + self.newline)
        if self.output_fd is None or self.options.tee:
            print(output, end=self.newline)

    def write(self, line, source):
        """Write the line either to the output file or stdout.

        This handles deciding whether to write to a file or print to standard
        out for subclasses. Override this if you want behaviour that differs
        from the default.

        :param str line:
            The formatted string to print or write.
        :param str source:
            The source code that has been formatted and associated with the
            line of output.
        """
        if line:
            self._write(line)
        if source:
            self._write(source)

    def stop(self):
        """Clean up after reporting is finished."""
        if self.output_fd is not None:
            self.output_fd.close()
            self.output_fd = None



"""Tests for flake8.plugins.manager.Plugin."""
import optparse

import mock
import pytest

from flake8 import exceptions
from flake8.plugins import manager


def test_load_plugin_fallsback_on_old_setuptools():
    """Verify we fallback gracefully to on old versions of setuptools."""
    entry_point = mock.Mock(spec=['load'])
    plugin = manager.Plugin('T000', entry_point)

    plugin.load_plugin()
    entry_point.load.assert_called_once_with(require=False)


def test_load_plugin_avoids_deprecated_entry_point_methods():
    """Verify we use the preferred methods on new versions of setuptools."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin = manager.Plugin('T000', entry_point)

    plugin.load_plugin(verify_requirements=True)
    assert entry_point.load.called is False
    entry_point.require.assert_called_once_with()
    entry_point.resolve.assert_called_once_with()


def test_load_plugin_is_idempotent():
    """Verify we use the preferred methods on new versions of setuptools."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin = manager.Plugin('T000', entry_point)

    plugin.load_plugin(verify_requirements=True)
    plugin.load_plugin(verify_requirements=True)
    plugin.load_plugin()
    assert entry_point.load.called is False
    entry_point.require.assert_called_once_with()
    entry_point.resolve.assert_called_once_with()


def test_load_plugin_only_calls_require_when_verifying_requirements():
    """Verify we do not call require when verify_requirements is False."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin = manager.Plugin('T000', entry_point)

    plugin.load_plugin()
    assert entry_point.load.called is False
    assert entry_point.require.called is False
    entry_point.resolve.assert_called_once_with()


def test_load_plugin_catches_and_reraises_exceptions():
    """Verify we raise our own FailedToLoadPlugin."""
    entry_point = mock.Mock(spec=['require', 'resolve'])
    entry_point.resolve.side_effect = ValueError('Test failure')
    plugin = manager.Plugin('T000', entry_point)

    with pytest.raises(exceptions.FailedToLoadPlugin):
        plugin.load_plugin()


def test_load_noncallable_plugin():
    """Verify that we do not load a non-callable plugin."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    entry_point.resolve.return_value = mock.NonCallableMock()
    plugin = manager.Plugin('T000', entry_point)

    with pytest.raises(exceptions.FailedToLoadPlugin):
        plugin.load_plugin()
    entry_point.resolve.assert_called_once_with()


def test_plugin_property_loads_plugin_on_first_use():
    """Verify that we load our plugin when we first try to use it."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin = manager.Plugin('T000', entry_point)

    assert plugin.plugin is not None
    entry_point.resolve.assert_called_once_with()


def test_execute_calls_plugin_with_passed_arguments():
    """Verify that we pass arguments directly to the plugin."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin_obj = mock.Mock()
    plugin = manager.Plugin('T000', entry_point)
    plugin._plugin = plugin_obj

    plugin.execute('arg1', 'arg2', kwarg1='value1', kwarg2='value2')
    plugin_obj.assert_called_once_with(
        'arg1', 'arg2', kwarg1='value1', kwarg2='value2'
    )

    # Extra assertions
    assert entry_point.load.called is False
    assert entry_point.require.called is False
    assert entry_point.resolve.called is False


def test_version_proxies_to_the_plugin():
    """Verify that we pass arguments directly to the plugin."""
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin_obj = mock.Mock(spec_set=['version'])
    plugin_obj.version = 'a.b.c'
    plugin = manager.Plugin('T000', entry_point)
    plugin._plugin = plugin_obj

    assert plugin.version == 'a.b.c'


def test_register_options():
    """Verify we call add_options on the plugin only if it exists."""
    # Set up our mocks and Plugin object
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin_obj = mock.Mock(spec_set=['name', 'version', 'add_options',
                                     'parse_options'])
    option_manager = mock.Mock(spec=['register_plugin'])
    plugin = manager.Plugin('T000', entry_point)
    plugin._plugin = plugin_obj

    # Call the method we're testing.
    plugin.register_options(option_manager)

    # Assert that we call add_options
    plugin_obj.add_options.assert_called_once_with(option_manager)


def test_register_options_checks_plugin_for_method():
    """Verify we call add_options on the plugin only if it exists."""
    # Set up our mocks and Plugin object
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin_obj = mock.Mock(spec_set=['name', 'version', 'parse_options'])
    option_manager = mock.Mock(spec=['register_plugin'])
    plugin = manager.Plugin('T000', entry_point)
    plugin._plugin = plugin_obj

    # Call the method we're testing.
    plugin.register_options(option_manager)

    # Assert that we register the plugin
    assert option_manager.register_plugin.called is False


def test_provide_options():
    """Verify we call add_options on the plugin only if it exists."""
    # Set up our mocks and Plugin object
    entry_point = mock.Mock(spec=['require', 'resolve', 'load'])
    plugin_obj = mock.Mock(spec_set=['name', 'version', 'add_options',
                                     'parse_options'])
    option_values = optparse.Values({'enable_extensions': []})
    option_manager = mock.Mock()
    plugin = manager.Plugin('T000', entry_point)
    plugin._plugin = plugin_obj

    # Call the method we're testing.
    plugin.provide_options(option_manager, option_values, None)

    # Assert that we call add_options
    plugin_obj.parse_options.assert_called_once_with(
        option_manager, option_values, None
    )


@pytest.mark.parametrize('ignore_list, code, expected_list', [
    (['E', 'W', 'F', 'C9'], 'W', ['E', 'F', 'C9']),
    (['E', 'W', 'F'], 'C9', ['E', 'W', 'F']),
])
def test_enable(ignore_list, code, expected_list):
    """Verify that enabling a plugin removes it from the ignore list."""
    options = mock.Mock(ignore=ignore_list)
    optmanager = mock.Mock()
    plugin = manager.Plugin(code, mock.Mock())

    plugin.enable(optmanager, options)

    assert options.ignore == expected_list


def test_enable_without_providing_parsed_options():
    """Verify that enabling a plugin removes it from the ignore list."""
    optmanager = mock.Mock()
    plugin = manager.Plugin('U4', mock.Mock())

    plugin.enable(optmanager)

    optmanager.remove_from_default_ignore.assert_called_once_with(['U4'])



"""Tests for the flake8.style_guide.DecisionEngine class."""
import optparse

import pytest

from flake8 import defaults
from flake8 import style_guide


def create_options(**kwargs):
    """Create and return an instance of optparse.Values."""
    kwargs.setdefault('select', [])
    kwargs.setdefault('extended_default_select', [])
    kwargs.setdefault('ignore', [])
    kwargs.setdefault('extend_ignore', [])
    kwargs.setdefault('disable_noqa', False)
    kwargs.setdefault('enable_extensions', [])
    return optparse.Values(kwargs)


@pytest.mark.parametrize('ignore_list,extend_ignore,error_code', [
    (['E111', 'E121'], [], 'E111'),
    (['E111', 'E121'], [], 'E121'),
    (['E111'], ['E121'], 'E121'),
    (['E11', 'E12'], [], 'E121'),
    (['E2', 'E12'], [], 'E121'),
    (['E2', 'E12'], [], 'E211'),
    (['E2', 'E3'], ['E12'], 'E211'),
])
def test_was_ignored_ignores_errors(ignore_list, extend_ignore, error_code):
    """Verify we detect users explicitly ignoring an error."""
    decider = style_guide.DecisionEngine(
        create_options(ignore=ignore_list, extend_ignore=extend_ignore))

    assert decider.was_ignored(error_code) is style_guide.Ignored.Explicitly


@pytest.mark.parametrize('ignore_list,extend_ignore,error_code', [
    (['E111', 'E121'], [], 'E112'),
    (['E111', 'E121'], [], 'E122'),
    (['E11', 'E12'], ['E121'], 'W121'),
    (['E2', 'E12'], [], 'E112'),
    (['E2', 'E12'], [], 'E111'),
    (['E2', 'E12'], ['W11', 'E3'], 'E111'),
])
def test_was_ignored_implicitly_selects_errors(ignore_list, extend_ignore,
                                               error_code):
    """Verify we detect users does not explicitly ignore an error."""
    decider = style_guide.DecisionEngine(
        create_options(ignore=ignore_list, extend_ignore=extend_ignore))

    assert decider.was_ignored(error_code) is style_guide.Selected.Implicitly


@pytest.mark.parametrize('select_list,enable_extensions,error_code', [
    (['E111', 'E121'], [], 'E111'),
    (['E111', 'E121'], [], 'E121'),
    (['E11', 'E12'], [], 'E121'),
    (['E2', 'E12'], [], 'E121'),
    (['E2', 'E12'], [], 'E211'),
    (['E1'], ['E2'], 'E211'),
    ([], ['E2'], 'E211'),
])
def test_was_selected_selects_errors(select_list, enable_extensions,
                                     error_code):
    """Verify we detect users explicitly selecting an error."""
    decider = style_guide.DecisionEngine(
        options=create_options(select=select_list,
                               enable_extensions=enable_extensions),
    )

    assert decider.was_selected(error_code) is style_guide.Selected.Explicitly


def test_was_selected_implicitly_selects_errors():
    """Verify we detect users implicitly selecting an error."""
    select_list = []
    error_code = 'E121'
    decider = style_guide.DecisionEngine(
        create_options(
            select=select_list,
            extended_default_select=['E'],
        ),
    )

    assert decider.was_selected(error_code) is style_guide.Selected.Implicitly


@pytest.mark.parametrize('select_list,error_code', [
    (['E111', 'E121'], 'E112'),
    (['E111', 'E121'], 'E122'),
    (['E11', 'E12'], 'E132'),
    (['E2', 'E12'], 'E321'),
    (['E2', 'E12'], 'E410'),
])
def test_was_selected_excludes_errors(select_list, error_code):
    """Verify we detect users implicitly excludes an error."""
    decider = style_guide.DecisionEngine(create_options(select=select_list))

    assert decider.was_selected(error_code) is style_guide.Ignored.Implicitly


@pytest.mark.parametrize(
    'select_list,ignore_list,extend_ignore,error_code,expected', [
        (['E111', 'E121'], [], [], 'E111', style_guide.Decision.Selected),
        (['E111', 'E121'], [], [], 'E112', style_guide.Decision.Ignored),
        (['E111', 'E121'], [], [], 'E121', style_guide.Decision.Selected),
        (['E111', 'E121'], [], [], 'E122', style_guide.Decision.Ignored),
        (['E11', 'E12'], [], [], 'E132', style_guide.Decision.Ignored),
        (['E2', 'E12'], [], [], 'E321', style_guide.Decision.Ignored),
        (['E2', 'E12'], [], [], 'E410', style_guide.Decision.Ignored),
        (['E11', 'E121'], ['E1'], [], 'E112', style_guide.Decision.Selected),
        (['E11', 'E121'], [], ['E1'], 'E112', style_guide.Decision.Selected),
        (['E111', 'E121'], ['E2'], ['E3'], 'E122',
         style_guide.Decision.Ignored),
        (['E11', 'E12'], ['E13'], [], 'E132', style_guide.Decision.Ignored),
        (['E1', 'E3'], ['E32'], [], 'E321', style_guide.Decision.Ignored),
        ([], ['E2', 'E12'], [], 'E410', style_guide.Decision.Ignored),
        (['E4'], ['E2', 'E12', 'E41'], [], 'E410',
         style_guide.Decision.Ignored),
        (['E41'], ['E2', 'E12', 'E4'], [], 'E410',
         style_guide.Decision.Selected),
        (['E'], ['F'], [], 'E410', style_guide.Decision.Selected),
        (['F'], [], [], 'E410', style_guide.Decision.Ignored),
        (['E'], defaults.IGNORE, [], 'E126', style_guide.Decision.Selected),
        (['W'], defaults.IGNORE, [], 'E126', style_guide.Decision.Ignored),
        (['E'], defaults.IGNORE, [], 'W391', style_guide.Decision.Ignored),
        (['E', 'W'], ['E13'], [], 'E131', style_guide.Decision.Ignored),
        (defaults.SELECT, ['E13'], [], 'E131', style_guide.Decision.Ignored),
        (defaults.SELECT, defaults.IGNORE, ['W391'], 'E126',
         style_guide.Decision.Ignored),
        (defaults.SELECT, defaults.IGNORE, [], 'W391',
         style_guide.Decision.Selected),
    ]
)
def test_decision_for(select_list, ignore_list, extend_ignore, error_code,
                      expected):
    """Verify we decide when to report an error."""
    decider = style_guide.DecisionEngine(
        create_options(select=select_list,
                       ignore=ignore_list,
                       extend_ignore=extend_ignore))

    assert decider.decision_for(error_code) is expected


@pytest.mark.parametrize(
    'select,ignore,extend_select,enabled_extensions,error_code,expected', [
        (defaults.SELECT, [], ['I1'], [], 'I100',
            style_guide.Decision.Selected),
        (defaults.SELECT, [], ['I1'], [], 'I201',
            style_guide.Decision.Ignored),
        (defaults.SELECT, ['I2'], ['I1'], [], 'I101',
            style_guide.Decision.Selected),
        (defaults.SELECT, ['I2'], ['I1'], [], 'I201',
            style_guide.Decision.Ignored),
        (defaults.SELECT, ['I1'], ['I10'], [], 'I101',
            style_guide.Decision.Selected),
        (defaults.SELECT, ['I10'], ['I1'], [], 'I101',
            style_guide.Decision.Ignored),
        (defaults.SELECT, [], [], ['U4'], 'U401',
            style_guide.Decision.Selected),
        (defaults.SELECT, ['U401'], [], ['U4'], 'U401',
            style_guide.Decision.Ignored),
        (defaults.SELECT, ['U401'], [], ['U4'], 'U402',
            style_guide.Decision.Selected),
        (['E', 'W'], ['E13'], [], [], 'E131', style_guide.Decision.Ignored),
        (['E', 'W'], ['E13'], [], [], 'E126', style_guide.Decision.Selected),
        (['E2'], ['E21'], [], [], 'E221', style_guide.Decision.Selected),
        (['E2'], ['E21'], [], [], 'E212', style_guide.Decision.Ignored),
        (['F', 'W'], ['C90'], ['I1'], [], 'C901',
            style_guide.Decision.Ignored),
        (['E', 'W'], ['C'], [], [], 'E131',
            style_guide.Decision.Selected),
        (defaults.SELECT, defaults.IGNORE, [], ['I'], 'I101',
            style_guide.Decision.Selected),
        (defaults.SELECT, defaults.IGNORE, ['G'], ['I'], 'G101',
            style_guide.Decision.Selected),
        (defaults.SELECT, ['G1'], ['G'], ['I'], 'G101',
            style_guide.Decision.Ignored),
        (defaults.SELECT, ['E126'], [], ['I'], 'I101',
            style_guide.Decision.Selected),
        (['E', 'W'], defaults.IGNORE, ['I'], [], 'I101',
            style_guide.Decision.Ignored),
        # TODO(sigmavirus24) Figure out how to exercise the final catch-all
        # return statement
    ]
)
def test_more_specific_decision_for_logic(select, ignore, extend_select,
                                          enabled_extensions, error_code,
                                          expected):
    """Verify the logic of DecisionEngine.more_specific_decision_for."""
    decider = style_guide.DecisionEngine(
        create_options(
            select=select, ignore=ignore,
            extended_default_select=extend_select,
            enable_extensions=enabled_extensions,
        ),
    )

    assert decider.more_specific_decision_for(error_code) is expected



"""Module containing shims around Flake8 2.x behaviour.

Previously, users would import :func:`get_style_guide` from ``flake8.engine``.
In 3.0 we no longer have an "engine" module but we maintain the API from it.
"""
import logging
import os.path

import flake8
from flake8.formatting import base as formatter
from flake8.main import application as app

LOG = logging.getLogger(__name__)


__all__ = ("get_style_guide",)


def get_style_guide(**kwargs):
    r"""Provision a StyleGuide for use.

    :param \*\*kwargs:
        Keyword arguments that provide some options for the StyleGuide.
    :returns:
        An initialized StyleGuide
    :rtype:
        :class:`StyleGuide`
    """
    application = app.Application()
    application.parse_preliminary_options_and_args([])
    flake8.configure_logging(
        application.prelim_opts.verbose, application.prelim_opts.output_file
    )
    application.make_config_finder()
    application.find_plugins()
    application.register_plugin_options()
    application.parse_configuration_and_cli([])
    # We basically want application.initialize to be called but with these
    # options set instead before we make our formatter, notifier, internal
    # style guide and file checker manager.
    options = application.options
    for key, value in kwargs.items():
        try:
            getattr(options, key)
            setattr(options, key, value)
        except AttributeError:
            LOG.error('Could not update option "%s"', key)
    application.make_formatter()
    application.make_notifier()
    application.make_guide()
    application.make_file_checker_manager()
    return StyleGuide(application)


class StyleGuide(object):
    """Public facing object that mimic's Flake8 2.0's StyleGuide.

    .. note::

        There are important changes in how this object behaves compared to
        the StyleGuide object provided in Flake8 2.x.

    .. warning::

        This object should not be instantiated directly by users.

    .. versionchanged:: 3.0.0
    """

    def __init__(self, application):
        """Initialize our StyleGuide."""
        self._application = application
        self._file_checker_manager = application.file_checker_manager

    @property
    def options(self):
        """Return application's options.

        An instance of :class:`optparse.Values` containing parsed options.
        """
        return self._application.options

    @property
    def paths(self):
        """Return the extra arguments passed as paths."""
        return self._application.paths

    def check_files(self, paths=None):
        """Run collected checks on the files provided.

        This will check the files passed in and return a :class:`Report`
        instance.

        :param list paths:
            List of filenames (or paths) to check.
        :returns:
            Object that mimic's Flake8 2.0's Reporter class.
        :rtype:
            flake8.api.legacy.Report
        """
        self._application.run_checks(paths)
        self._application.report_errors()
        return Report(self._application)

    def excluded(self, filename, parent=None):
        """Determine if a file is excluded.

        :param str filename:
            Path to the file to check if it is excluded.
        :param str parent:
            Name of the parent directory containing the file.
        :returns:
            True if the filename is excluded, False otherwise.
        :rtype:
            bool
        """
        return self._file_checker_manager.is_path_excluded(filename) or (
            parent
            and self._file_checker_manager.is_path_excluded(
                os.path.join(parent, filename)
            )
        )

    def init_report(self, reporter=None):
        """Set up a formatter for this run of Flake8."""
        if reporter is None:
            return
        if not issubclass(reporter, formatter.BaseFormatter):
            raise ValueError(
                "Report should be subclass of "
                "flake8.formatter.BaseFormatter."
            )
        self._application.formatter = None
        self._application.make_formatter(reporter)
        self._application.guide = None
        # NOTE(sigmavirus24): This isn't the intended use of
        # Application#make_guide but it works pretty well.
        # Stop cringing... I know it's gross.
        self._application.make_guide()
        self._application.file_checker_manager = None
        self._application.make_file_checker_manager()

    def input_file(self, filename, lines=None, expected=None, line_offset=0):
        """Run collected checks on a single file.

        This will check the file passed in and return a :class:`Report`
        instance.

        :param str filename:
            The path to the file to check.
        :param list lines:
            Ignored since Flake8 3.0.
        :param expected:
            Ignored since Flake8 3.0.
        :param int line_offset:
            Ignored since Flake8 3.0.
        :returns:
            Object that mimic's Flake8 2.0's Reporter class.
        :rtype:
            flake8.api.legacy.Report
        """
        return self.check_files([filename])


class Report(object):
    """Public facing object that mimic's Flake8 2.0's API.

    .. note::

        There are important changes in how this object behaves compared to
        the object provided in Flake8 2.x.

    .. warning::

        This should not be instantiated by users.

    .. versionchanged:: 3.0.0
    """

    def __init__(self, application):
        """Initialize the Report for the user.

        .. warning:: This should not be instantiated by users.
        """
        self._application = application
        self._style_guide = application.guide
        self._stats = self._style_guide.stats

    @property
    def total_errors(self):
        """Return the total number of errors."""
        return self._application.result_count

    def get_statistics(self, violation):
        """Get the list of occurrences of a violation.

        :returns:
            List of occurrences of a violation formatted as:
            {Count} {Error Code} {Message}, e.g.,
            ``8 E531 Some error message about the error``
        :rtype:
            list
        """
        return [
            "{} {} {}".format(s.count, s.error_code, s.message)
            for s in self._stats.statistics_for(violation)
        ]
